
/**
 * Discard UI tutorial
 *
 * @param screenGameWorld   {APP.ScreenGameWorld}
 */
function tutorialUiDiscard( screenGameWorld ){
    var overlayMan  = screenGameWorld.overlayMan;
    screenGameWorld.stepQueue(
        function(){
            // Wait a bit...
            screenGameWorld.getScene().createTimer( screenGameWorld.time + 500, 0, this );
        },
        function(){
            overlayMan.queueDialog({
                widthMaxPct: 0.7,
                title: "The Discard Panel",
                buttons: [ "ok", "Got It!" ],
                content: [
                    {
                        content: "You may discard cards you don’t need when you end your turn.",
                        padding: 15
                    },
                    "To discard a card, drag it over the trash zone and release. When you're ready, touch the 'Go!' button to REALLY end your turn."
                ],
                closeFn: this
            });
        }
    );
}

var campaign = {
    id:             "tutorial",
    title:          "Tutorial",
    desc:           [
                        "Welcome to basic training, cadet! It's time to put your noodle to work.",
                        "",
                        "Remember: winners never quit, and quitters never collect the completion reward."
                    ],
    winMsg:         "Stick a fork in it... it's D-O-N-E! Congratulations on beating the tutorial.",
    difficulty:     0.5,
    missionsMin:    3,

    reqUnlocks: null,
    lockDesc:   "",

    rewards: function( completeCount ) {
        if( completeCount > 0 )
            return null;

        return {
            "tokens":   1000,
            "unlocks":  [ "practice", "squared", "easy" ]
        }
    },

    challenges: [
        {
            "id":           "tut1",
            "title":        "The Fundamentals",
            "desc":         "An must-play introduction to the game basics. Yawning is optional...",
            "difficulty":   0.5,
            "playCost":     0,
            "dropCost":     -1,

            "map":          "squared",
            "deck":         [
                { "card": "deployown1",       "count": 4    },
                { "card": "deployown3",       "count": 1    }
            ],
            "winTributes":  3,

            "allowFavors":  false,

            "winReward":    null,
            "loseReward":   null,

            "reqRankMin":   0,
            "reqRankMax":   0,

            "configCustom": function( gameLogic ) {
                gameLogic.configureRegionReveal( 0, {credits: 0, tributes: 1} );
                gameLogic.configureRegionReveal( 1, {credits: -1, tributes: 2} );
                gameLogic.configureRegionReveal( 2, {credits: -1, tributes: 2} );
                gameLogic.configureRegionReveal( 3, {credits: 0, tributes: 1} );
            },

            "noSave": true,

            "storyHandler": {
                preEventActive: null,
                postEventActive: null,

                /**
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                init: function( screenGameWorld ) {
                    this.setEventHandler( null, this.postEventTutorial1 );
                    screenGameWorld.hud.buttonSubmit.setEnabled( false );
                },

                /**
                 * @param gameEvt           {APP.REFS.GameSession.Event|APP.REFS.GameSession.EventTurnStart}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                preEvent: function( gameEvt, screenGameWorld ) {
                    if( this.preEventActive == null )
                        return;

                    this.preEventActive( gameEvt, screenGameWorld );
                },

                /**
                 * @param gameEvt           {APP.REFS.GameSession.Event|APP.REFS.GameSession.EventTurnStart}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                postEvent: function( gameEvt, screenGameWorld ) {
                    if( this.postEventActive == null )
                        return;

                    this.postEventActive( gameEvt, screenGameWorld );
                },

                /**
                 * Set the event handler
                 * @private
                 *
                 * @param preEvent  {function(evt:APP.REFS.GameSession.Event, screen:APP.ScreenGameWorld)}
                 * @param postEvent {function(evt:APP.REFS.GameSession.Event, screen:APP.ScreenGameWorld)}
                 */
                setEventHandler: function( preEvent, postEvent ) {
                    this.preEventActive     = preEvent;
                    this.postEventActive    = postEvent;
                },

                /**
                 * Post event handler (intro)
                 * @param gameEvt           {APP.REFS.GameSession.EventPlayerAction}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                postEventTutorial1: function( gameEvt, screenGameWorld ) {
                    if( screenGameWorld.isGameOver() )
                        return;

                    if( gameEvt.type != "PlayerAction" )
                        return;
                    if( gameEvt.action != "turnStart" )
                        return;
                    if( gameEvt.playerIdx != screenGameWorld.localPlayerIdx )
                        return;

                    this.setEventHandler( null, null );

                    var overlayMan  = screenGameWorld.overlayMan,
                        me          = this;
                    screenGameWorld.stepQueue( function(){
                        overlayMan.queueDialog({
                            widthMaxPct: 0.5,
                            title: "Welcome!",
                            buttons: [ "ok", "Sir, Yes Sir!" ],
                            content: [
                                {
                                    padding:    15,
                                    content:    "Welcome to the training grounds!"
                                },
                                "You won't be competing against a rival team so failure is NOT an option."
                            ]
                        });
                        overlayMan.queueDialog({
                            widthMaxPct: 0.6,
                            title: "The Details",
                            buttons: [ "ok", "Got It!" ],
                            content: {
                                align:   0.5,
                                content: screenGameWorld.createImageDynamic( "images/tut_goals.png" )
                                    .setPositionAnchor( 0.5, 0.5 )
                                    .setSize( 330, 190 )
                            },
                            closeFn: function(){
                                // Prevent cards from being available...
                                me.setEventHandler( null, function(){
                                    screenGameWorld.stepQueue( function(){
                                        screenGameWorld.hud.handMan.setCardsAvailableState( "never" );
                                        this();
                                    })
                                } );
                                screenGameWorld.world.collectibleMan.eventOnce( "collectedAll", function(){
                                    me.setEventHandler( null, null );
                                    screenGameWorld.hud.handMan.setCardsAvailableState( "normal" );
                                    me.tutorial2( screenGameWorld );
                                } );
                                this();
                            },
                            closeCtx: this
                        });
                    });
                },

                /**
                 * Tutorial (Mechanics)
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                tutorial2: function( screenGameWorld ) {
                    if( screenGameWorld.isGameOver() )
                        return;

                    var overlayMan  = screenGameWorld.overlayMan,
                        me          = this;
                    screenGameWorld.stepQueue( function(){
                        overlayMan.queueDialog({
                            widthMaxPct: 0.85,
                            title: "Getting Gluons",
                            buttons: [ "ok", "A ha!" ],
                            content: {
                                align:   0.5,
                                content: screenGameWorld.createImageDynamic( "images/tut_collecttrib.png" )
                                    .setPositionAnchor( 0.5, 0.5 )
                                    .setSize( 340, 120 )
                            }
                        });
                        overlayMan.queueDialog({
                            widthMaxPct: 0.5,
                            title: "Deploy Grunts!",
                            buttons: [ "ok", "Got It!" ],
                            content: [
                                {
                                    content: "Use 'Grunt' cards to deploy grunts in a region.",
                                    padding: 15
                                },
                                {
                                    align:   0.5,
                                    content: screenGameWorld.createImageDynamic( "images/tut_card.png" )
                                        .setPositionAnchor( 0.5, 0.5 )
                                        .setSize( 250, 200 )
                                }
                            ],
                            closeFn: function(){
                                me.uiStart = function( uiName, screenGameWorld ) {
                                    if( uiName == "ui_card" && screenGameWorld.cardAction.type == "deploy" ) {
                                        screenGameWorld.stepQueue(
                                            function(){
                                                // Wait a bit...
                                                screenGameWorld.getScene().createTimer( screenGameWorld.time + 500, 0, this );
                                            },
                                            function(){
                                                overlayMan.queueDialog({
                                                    widthMaxPct: 0.7,
                                                    title: "Issuing Orders",
                                                    buttons: [ "ok", "Got It!" ],
                                                    content: "The planner screen is the last step in issuing orders. If you're not sure what to do, touch the '?' in the top left corner to get help.",
                                                    closeFn: function(){
                                                        screenGameWorld.hud.buttonSubmit.setEnabled( true );
                                                        me.setEventHandler( null, me.postEventTutorial3 );
                                                        this();
                                                    },
                                                    closeCtx: this
                                                });
                                            } );
                                        me.uiStart = undefined;
                                    }
                                };
                                this();
                            },
                            closeCtx: this
                        });
                    });
                },

                /**
                 * Post event tutorial handler (deploy UI)
                 * @param gameEvt           {APP.REFS.GameSession.EventHandChange}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                postEventTutorial3: function( gameEvt, screenGameWorld ) {
                    if( screenGameWorld.isGameOver() )
                        return;

                    if( gameEvt.type != "Deploy" )
                        return;

                    this.setEventHandler( null, null );

                    var overlayMan  = screenGameWorld.overlayMan,
                        me          = this;
                    screenGameWorld.stepQueue(
                        function(){
                            // Wait a bit...
                            screenGameWorld.getScene().createTimer( screenGameWorld.time + 1000, 0, this );
                        },
                        function(){
                            overlayMan.queueDialog({
                                widthMaxPct: 0.7,
                                title: "Region Claimed!",
                                buttons: [ "ok", "Booyah!" ],
                                content: [
                                    {
                                        content: "Well done! You've deployed grunts and claimed a region.",
                                        padding: 10
                                    },
                                    {
                                        content: "Reapers are automatically deployed into claimed regions and show ownership. A region always belongs to the team with the largest grunt presence in it.",
                                        padding: 25
                                    },
                                    {
                                        align:   0.5,
                                        content: screenGameWorld.createImageDynamic( "images/tut_harvester.png" )
                                            .setPositionAnchor( 0.5, 0.5 )
                                            .setSize( 200, 100 )
                                    }
                                ]
                            });
                            overlayMan.queueDialog({
                                widthMaxPct: 0.6,
                                title: "Reaping",
                                buttons: [ "ok", "Okaaay" ],
                                content: [
                                    {
                                        content: "Reapers takes one turn to extract a region's resources so let's move on to the next turn.",
                                        padding: 15
                                    },
                                    "Touch the 'Go!' button to end the current turn."
                                ],
                                closeFn: function(){
                                    var showTutorialUiDiscard = true;
                                    me.uiStart = function( uiName, screenGameWorld ) {
                                        if( uiName == "ui_endturn" ) {
                                            me.uiStart = null;
                                            if( showTutorialUiDiscard ) {
                                                tutorialUiDiscard.call( this, screenGameWorld );
                                                showTutorialUiDiscard = false;
                                            }

                                            // The end turn game event signals the end of this tutorial...
                                            me.setEventHandler( null, function(gameEvt, screenGameWorld){
                                                if( gameEvt.type != "TurnEnd" )
                                                    return;

                                                me.setEventHandler( null, me.postEventTutorial4 );
                                            } );
                                        }
                                    };
                                    this();
                                },
                                closeCtx: this
                            });
                        }
                    );
                },

                /**
                 * Post event tutorial handler (reap)
                 * @param gameEvt           {APP.REFS.GameSession.EventHandChange}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                postEventTutorial4: function( gameEvt, screenGameWorld ) {
                    if( screenGameWorld.isGameOver() )
                        return;

                    if( gameEvt.type != "TurnStart" )
                        return;
                    if( gameEvt.playerIdx != screenGameWorld.localPlayerIdx )
                        return;

                    this.setEventHandler( null, null );

                    var overlayMan  = screenGameWorld.overlayMan,
                        me          = this;
                    screenGameWorld.stepQueue(
                        function(){
                            // Wait a bit...
                            screenGameWorld.getScene().createTimer( screenGameWorld.time + 1500, 0, this );
                        },
                        function(){
                            overlayMan.queueDialog({
                                widthMaxPct: 0.6,
                                title: "Reaping",
                                buttons: [ "ok", "Let's Do It!" ],
                                content: [
                                    {
                                        content: "When a reaper is ready, it glows. Its activation cost and output depends on the region.",
                                        padding: 10
                                    },
                                    {
                                        align:   0.5,
                                        padding: 10,
                                        content: screenGameWorld.createImageDynamic( "images/tut_regionsignboard.png" )
                                            .setPositionAnchor( 0.5, 0.5 )
                                            .setSize( 250, 120 )
                                    },
                                    "Touch a brightly glowing reaper to activate it."
                                ],
                                closeFn: function(){
                                    me.setEventHandler( null, function( gameEvt, screenGameWorld ){
                                        if( gameEvt.type != "Harvest" )
                                            return;
                                        if( gameEvt.playerIdx != screenGameWorld.localPlayerIdx )
                                            return;

                                        me.setEventHandler( null, null );
                                        me.tutorial5( screenGameWorld );
                                    } );
                                    this();
                                },
                                closeCtx: this
                            });
                        }
                    );
                },

                /**
                 * Post event tutorial handler (conclusion)
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                tutorial5: function( screenGameWorld ) {
                    if( screenGameWorld.isGameOver() )
                        return;

                    var overlayMan  = screenGameWorld.overlayMan,
                        me          = this;
                    screenGameWorld.stepQueue(
                        function(){
                            // Wait a bit...
                            screenGameWorld.getScene().createTimer( screenGameWorld.time + 2000, 0, this );
                        },
                        function(){
                            overlayMan.queueDialog({
                                widthMaxPct: 0.6,
                                title: "Reaped!",
                                buttons: [ "ok", "Yeah!" ],
                                content: "Well done, You’ve successfully collected gluons!"
                            });
                            overlayMan.queueDialog({
                                widthMaxPct: 0.7,
                                title: "Mission Status",
                                buttons: [ "ok", "Gotcha!" ],
                                content: [
                                    {
                                        content: "The player panel at the bottom of the screen keeps track of the mission and player status.",
                                        padding: 15
                                    },
                                    {
                                        align:   0.5,
                                        padding: 10,
                                        content: screenGameWorld.createImageDynamic( "images/tut_panel.png" )
                                            .setPositionAnchor( 0.5, 0.5 )
                                            .setSize( 410, 150 )
                                    },
                                    "Make sure that you pay attention to it!"
                                ]
                            });
                            overlayMan.queueDialog({
                                widthMaxPct: 0.6,
                                title: "Next Steps",
                                buttons: [ "ok", "Sir, Yes Sir!" ],
                                content: "We've covered the basics. Now, go apply that knowledge and finish up this mission!",
                                closeFn: this
                            });
                        }
                    );
                }
            }
        },
        {
            "id":           "tut2",
            "title":        "Getting Around",
            "desc":         "Learn about moving troops and exploring regions",
            "difficulty":   0.5,
            "playCost":     0,
            "dropCost":     -1,

            "map":          "squared",
            "deck":         [
                { "card": "moveown1",         "count": 4    },
                { "card": "moveown3",         "count": 1    }
            ],
            "winTributes":  5,
            "regionValHeap": [
                { "credits": 0, "tributes": 1   },
                { "credits": -1, "tributes": 2  },
                { "credits": -1, "tributes": 2  }
            ],

            "allowFavors":  false,

            "winReward":    null,
            "loseReward":   null,

            "reqRankMin":   1,
            "reqRankMax":   1,

            "configCustom": function( gameLogic ) {
                gameLogic.configureRegionReveal( 0, {credits: 1, tributes: 0} );
                gameLogic.configureAddDeploys( 0, 0, 5 );
            },

            "noSave": true,

            "storyHandler": {
                preEventActive: null,
                postEventActive: null,

                /**
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                init: function( screenGameWorld ) {
                    this.setEventHandler( null, this.postEventTutorial1 );
                },

                /**
                 * @param gameEvt           {APP.REFS.GameSession.Event|APP.REFS.GameSession.EventTurnStart}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                preEvent: function( gameEvt, screenGameWorld ) {
                    if( this.preEventActive == null )
                        return;

                    this.preEventActive( gameEvt, screenGameWorld );
                },

                /**
                 * @param gameEvt           {APP.REFS.GameSession.Event|APP.REFS.GameSession.EventTurnStart}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                postEvent: function( gameEvt, screenGameWorld ) {
                    if( this.postEventActive == null )
                        return;

                    this.postEventActive( gameEvt, screenGameWorld );
                },

                /**
                 * Set the event handler
                 * @private
                 *
                 * @param preEvent  {function(evt:APP.REFS.GameSession.Event, screen:APP.ScreenGameWorld)}
                 * @param postEvent {function(evt:APP.REFS.GameSession.Event, screen:APP.ScreenGameWorld)}
                 */
                setEventHandler: function( preEvent, postEvent ) {
                    this.preEventActive     = preEvent;
                    this.postEventActive    = postEvent;
                },

                /**
                 * Post event handler (intro)
                 * @param gameEvt           {APP.REFS.GameSession.EventPlayerAction}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                postEventTutorial1: function( gameEvt, screenGameWorld ) {
                    if( screenGameWorld.isGameOver() )
                        return;

                    if( gameEvt.type != "PlayerAction" )
                        return;
                    if( gameEvt.action != "turnStart" )
                        return;
                    if( gameEvt.playerIdx != screenGameWorld.localPlayerIdx )
                        return;

                    this.setEventHandler( null, null );

                    var overlayMan  = screenGameWorld.overlayMan,
                        me          = this;
                    screenGameWorld.stepQueue( function(){
                        overlayMan.queueDialog({
                            widthMaxPct: 0.6,
                            title: "Introduction",
                            buttons: [ "ok", "Got It!" ],
                            content: "Welcome to the second tutorial. This time, you'll only have invasion cards to work with. Use them to take over the entire map."
                        });
                        overlayMan.queueDialog({
                            widthMaxPct: 0.6,
                            title: "Invasion/Exploration",
                            buttons: [ "ok", "Sir, Yes Sir!" ],
                            content: [
                                {
                                    padding: 10,
                                    content: "The map you're invading is filled with unexplored regions."
                                },
                                {
                                    align:   0.5,
                                    padding: 10,
                                    content: screenGameWorld.createImageDynamic( "images/tut_regioninfo.png" )
                                        .setPositionAnchor( 0.5, 0.5 )
                                        .setSize( 200, 80 )
                                },
                                "Invade an unexplored region to claim it and reveal its resources."
                            ],
                            closeFn: function(){
                                me.setEventHandler( null, me.postEventTutorial2 );
                                this();
                            },
                            closeCtx: this
                        });
                    });
                },

                /**
                 * Post event handler (intro)
                 * @param gameEvt           {APP.REFS.GameSession.EventPlayerAction}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                postEventTutorial2: function( gameEvt, screenGameWorld ) {
                    if( screenGameWorld.isGameOver() )
                        return;

                    if( gameEvt.type != "RevealRegion" )
                        return;
                    if( gameEvt.playerIdx != screenGameWorld.localPlayerIdx )
                        return;

                    this.setEventHandler( null, null );

                    var overlayMan  = screenGameWorld.overlayMan,
                        me          = this;
                    screenGameWorld.stepQueue(
                        function(){
                            // Wait a bit...
                            screenGameWorld.getScene().createTimer( screenGameWorld.time + 1500, 0, this );
                        },
                        function(){
                            overlayMan.queueDialog({
                                widthMaxPct: 0.6,
                                title: "Invasion!",
                                buttons: [ "ok", "Sir, Yes Sir!" ],
                                content: [
                                    {
                                        padding: 10,
                                        content: "Well done! You've invaded and explored a region."
                                    },
                                    "Go ahead and keep practicing. Just remember, you'll need to COLLECT GLUONS to complete the mission!"
                                ],
                                closeFn: this
                            });
                        }
                    );
                }
            }
        },
        {
            "id":               "tut3",
            "title":            "Practices With Bots",
            "desc":             "Practice everything you've learnt so far. Against a bot! Using a complete deck of cards!!",
            "difficulty":       0.5,
            "playCost":         0,
            "dropCost":         -1,

            "map":              "squared",
            "ai":               "__ai_trainer",
            "winTributes":      5,
            "startPlayerIdx":   0,

            "regionValHeap": {
                "0": { "credits": 1,    "tributes": 0  },
                "1": { "credits": 0,    "tributes": 1  },
                "2": { "credits": -1,   "tributes": 2  },
                "3": { "credits": 0,    "tributes": 1  }
            },

            "allowFavors":  false,
            "winReward":    null,
            "loseReward":   null,

            "reqRankMin":   2,
            "reqRankMax":   Number.POSITIVE_INFINITY,

            "configCustom": function( gameLogic ) {
                gameLogic.configureRegionReveal( 0 );
                gameLogic.configureAddDeploys( 1, 1, 1 );
                gameLogic.configureAddDeploys( 2, 1, 3 );
                gameLogic.configureAddDeploys( 3, 1, 1 );
            },

            "noSave": true,

            "storyHandler": {
                preEventActive: null,
                postEventActive: null,

                init: function( gameProcessor ) {
                    this.setEventHandler( null, this.postEventTutorial1 );
                },

                /**
                 * @param gameEvt           {APP.REFS.GameSession.Event|APP.REFS.GameSession.EventTurnStart}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                preEvent: function( gameEvt, screenGameWorld ) {
                    if( this.preEventActive == null )
                        return;

                    this.preEventActive( gameEvt, screenGameWorld );
                },

                /**
                 * @param gameEvt           {APP.REFS.GameSession.Event|APP.REFS.GameSession.EventTurnStart}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                postEvent: function( gameEvt, screenGameWorld ) {
                    if( this.postEventActive == null )
                        return;

                    this.postEventActive( gameEvt, screenGameWorld );
                },

                /**
                 * Set the event handler
                 * @private
                 *
                 * @param preEvent  {function(evt:APP.REFS.GameSession.Event, screen:APP.ScreenGameWorld)}
                 * @param postEvent {function(evt:APP.REFS.GameSession.Event, screen:APP.ScreenGameWorld)}
                 */
                setEventHandler: function( preEvent, postEvent ) {
                    this.preEventActive     = preEvent;
                    this.postEventActive    = postEvent;
                },

                /**
                 * Post event handler (intro)
                 * @param gameEvt           {APP.REFS.GameSession.EventPlayerAction}
                 * @param screenGameWorld   {APP.ScreenGameWorld}
                 */
                postEventTutorial1: function( gameEvt, screenGameWorld ) {
                    if( screenGameWorld.isGameOver() )
                        return;

                    if( gameEvt.type != "PlayerAction" )
                        return;
                    if( gameEvt.action != "turnStart" )
                        return;
                    if( gameEvt.playerIdx != screenGameWorld.localPlayerIdx )
                        return;

                    this.setEventHandler( null, null );

                    var overlayMan  = screenGameWorld.overlayMan,
                        me          = this;
                    screenGameWorld.stepQueue( function(){
                        overlayMan.queueDialog({
                            widthMaxPct: 0.7,
                            title: "Introduction",
                            buttons: [ "ok", "*Groan*" ],
                            content: [
                                {
                                    content: "Welcome to the graduation test! This time, you'll be facing a trainer bot.",
                                    padding: 10
                                },
                                {
                                    content: "The good news is that you'll be playing with a full deck of cards... including rival team manipulation cards. The bad news is that we've allowed the bot to deploy ahead of time.",
                                    padding: 10
                                },
                                "Enough talk! Let's see you handle a little competition. Good luck."
                            ],
                            closeFn: this
                        });
                    });
                }
            }
        }
    ]
};

campaign;